from enoslib.api import run_command, wait_ssh
from enoslib.infra.enos_g5k.provider import G5k
from enoslib.infra.enos_g5k.configuration import Configuration, NetworkConfiguration
from enoslib.service import Monitoring

import logging


def pprint(d):
    """Utils fonction to pretty print the results"""
    for k, v in d["ok"].items():
        print(f"Result for {k}")
        print("-" * 70)
        print("STDOUT:")
        print(v.get("stdout", ""))
        print("STDERR:")
        print(v.get("stderr", ""))


logging.basicConfig(level=logging.INFO)

######################################################################
##
## Some parameters.
## Note 1: that you don't need to be on rennes frontend to use nodes
##         from rennes
## Note 2: Adapt the site/cluster according to the availibility
##         see the Gantt in https://www.grid5000.fr/w/Status
######################################################################

SITE = "rennes"
CLUSTER = "paravance"

######################################################################
##
## Configuration object describes the resource we want
## here: 2 machines on the same cluster using the production network
##
######################################################################
network = NetworkConfiguration(id="n1",
                               type="prod",
                               roles=["my_network"],
                               site=SITE)

conf = Configuration.from_settings(job_name="enoslib_tutorial",
                                   job_type="allow_classic_ssh")\
    .add_network_conf(network)\
    .add_machine(roles=["server"],
                 cluster=CLUSTER,
                 nodes=1,
                 primary_network=network)\
    .add_machine(roles=["client"],
                 cluster=CLUSTER,
                 nodes=1,
                 primary_network=network)\
    .finalize()

######################################################################
##
## Reserve the ressources corresponding to the configuration
##  you'll get two **physical machine** (not virtual)
## the roles object is a dictionnary of the concrete compute resources
## roles = {"server": [host1], "client": [host2] }
##
######################################################################

provider = G5k(conf)
roles, networks =  provider.init()
wait_ssh(roles)


######################################################################
##
## This deploys a monitoring stack. It is composed of
## - some agents on each monitored nodes
## - one collector that collects the metrics from the agents
## - one UI to visualize
##
######################################################################

m = Monitoring(collector=roles["server"],
               agent=roles["server"] + roles["client"],
               ui=roles["server"])
m.deploy()


######################################################################
##
## Below is the experimentation logic
##  - It installs the bare minimum to run iperf3
##  - The machine with the role 'server' is used to run a iperf3 server
##    started in the background (using tmux)
##  - The machine with the role 'client' connects to that server and initiate a
##    transfer for 600s (duration variable)
##  - Report is printed in stdout
######################################################################

server = roles["server"][0]
duration = 600
run_command("apt update && apt install -y iperf3 tmux", roles=roles)
run_command("tmux new-session -d 'exec iperf3 -s'", pattern_hosts="server", roles=roles)
result = run_command(f"iperf3 -c {server.address} -t {duration}", pattern_hosts="client", roles=roles)
pprint(result)

######################################################################
##
## Destroy the reservation, uncomment when needed
##
######################################################################

# provider.destroy()
